# Copyright 2008-2009 WebDriver committers
# Copyright 2008-2009 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Exceptions that may happen in all the webdriver code."""
class WebDriverException(Exception):
    def __init__(self, msg=None):
        Exception.__init__(self, msg)

class ErrorInResponseException(WebDriverException):
    """An error has occurred on the server side.

    This may happen when communicating with the firefox extension
    or the remote driver server."""
    def __init__(self, response, msg):
        WebDriverException.__init__(self, msg)
        self.response = response

class InvalidSwitchToTargetException(WebDriverException):
    """The frame or window target to be switched doesn't exist."""
    pass

class NoSuchFrameException(InvalidSwitchToTargetException):
    pass

class NoSuchWindowException(InvalidSwitchToTargetException):
    pass

class NoSuchElementException(WebDriverException):
    """find_element_by_* can't find the element."""
    pass

class NoSuchAttributeException(WebDriverException):
    """find_element_by_* can't find the element."""
    pass

class StaleElementReferenceException(WebDriverException):
    pass

class ElementNotVisibleException(WebDriverException):
    pass

class InvalidElementStateException(WebDriverException):
    pass

class ElementNotSelectableException(WebDriverException):
    pass

class InvalidCookieDomainException(WebDriverException):
    pass

class UnableToSetCookieException(WebDriverException):
    pass

class RemoteDriverServerException(WebDriverException):
    pass

class TimeoutException(WebDriverException):
    pass



# Copyright 2010 WebDriver committers
# Copyright 2010 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

class Command(object):
    """Defines constants for the standard WebDriver commands.

    While these constants have no meaning in and of themselves, they are
    used to marshal commands through a service that implements WebDriver's
    remote wire protocol:
        http://code.google.com/p/selenium/wiki/JsonWireProtocol
    """

    # Keep in sync with org.openqa.selenium.remote.DriverCommand

    NEW_SESSION = "newSession"
    DELETE_SESSION = "deleteSession"
    CLOSE = "close"
    QUIT = "quit"
    GET = "get"
    GO_BACK = "goBack"
    GO_FORWARD = "goForward"
    REFRESH = "refresh"
    ADD_COOKIE = "addCookie"
    GET_COOKIE = "getCookie"
    GET_ALL_COOKIES = "getCookies"
    DELETE_COOKIE = "deleteCookie"
    DELETE_ALL_COOKIES = "deleteAllCookies"
    FIND_ELEMENT = "findElement"
    FIND_ELEMENTS = "findElements"
    FIND_CHILD_ELEMENT = "findChildElement"
    FIND_CHILD_ELEMENTS = "findChildElements"
    CLEAR_ELEMENT = "clearElement"
    CLICK_ELEMENT = "clickElement"
    HOVER_OVER_ELEMENT = "hoverOverElement"
    SEND_KEYS_TO_ELEMENT = "sendKeysToElement"
    SUBMIT_ELEMENT = "submitElement"
    TOGGLE_ELEMENT = "toggleElement"
    GET_CURRENT_WINDOW_HANDLE = "getCurrentWindowHandle"
    GET_WINDOW_HANDLES = "getWindowHandles"
    SWITCH_TO_WINDOW = "switchToWindow"
    SWITCH_TO_FRAME = "switchToFrame"
    GET_ACTIVE_ELEMENT = "getActiveElement"
    GET_CURRENT_URL = "getCurrentUrl"
    GET_PAGE_SOURCE = "getPageSource"
    GET_TITLE = "getTitle"
    EXECUTE_SCRIPT = "executeScript"
    GET_SPEED = "getSpeed"
    SET_SPEED = "setSpeed"
    SET_BROWSER_VISIBLE = "setBrowserVisible"
    IS_BROWSER_VISIBLE = "isBrowserVisible"
    GET_ELEMENT_TEXT = "getElementText"
    GET_ELEMENT_VALUE = "getElementValue"
    GET_ELEMENT_TAG_NAME = "getElementTagName"
    SET_ELEMENT_SELECTED = "setElementSelected"
    DRAG_ELEMENT = "dragElement"
    IS_ELEMENT_SELECTED = "isElementSelected"
    IS_ELEMENT_ENABLED = "isElementEnabled"
    IS_ELEMENT_DISPLAYED = "isElementDisplayed"
    GET_ELEMENT_LOCATION = "getElementLocation"
    GET_ELEMENT_LOCATION_ONCE_SCROLLED_INTO_VIEW = (
        "getElementLocationOnceScrolledIntoView")
    GET_ELEMENT_SIZE = "getElementSize"
    GET_ELEMENT_ATTRIBUTE = "getElementAttribute"
    GET_ELEMENT_VALUE_OF_CSS_PROPERTY = "getElementValueOfCssProperty"
    ELEMENT_EQUALS = "elementEquals"
    SCREENSHOT = "screenshot"
    IMPLICIT_WAIT = "implicitlyWait"
    EXECUTE_ASYNC_SCRIPT = "executeAsyncScript"
    SET_SCRIPT_TIMEOUT = "setScriptTimeout"
    GET_ELEMENT_VALUE_OF_CSS_PROPERTY = "getElementValueOfCssProperty"

    #Alerts
    DISMISS_ALERT = "dismissAlert"
    ACCEPT_ALERT = "acceptAlert"
    SET_ALERT_VALUE = "setAlertValue"
    GET_ALERT_TEXT = "getAlertText"



# Copyright 2008-2010 WebDriver committers
# Copyright 2008-2010 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import with_statement
import os
from selenium.webdriver.common.exceptions import RemoteDriverServerException
from selenium.webdriver.remote import utils
import subprocess
import httplib
import BaseHTTPServer
import SimpleHTTPServer
import threading
import Queue
import time
import urllib
import sys
import platform
import tempfile
import shutil
import signal
try:
    import json
except ImportError:
    import simplejson as json

INITIAL_HTML = '''
<html>
    <head>
    <script type='text/javascript'>
            if (window.location.search == '') {
                setTimeout("window.location = window.location.href + '?reloaded'", 5000);
            }
    </script>
    </head>
    <body>
        <p>
            ChromeDriver server started and connected.  Please leave this tab open.
        </p>
    </body>
</html>'''

class RequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
    def do_GET(self):
        self.respond(INITIAL_HTML, "text/html")
    # From my (Miki Tebeka) understanding, the driver works by sending a POST http
    # request to the server, the server replies with the command to execute and the
    # next POST reply will have the result. So we hold a command and result
    # queues on the server. When we get a POST request we see if there is a
    # response in there and place it in the result queue, then we pop the next
    # command from the command queue (blocking if needed) the post it to the
    # client.
    # FIXME: Somewhere here there is a race condition I'm still hunting.
    #        One option to try is http://blog.odegra.com/?p=3

    def do_POST(self):
        self.process_reply()
        command = self.server.command_queue.get()
        data = json.dumps(command)
        self.respond(data, "application/json")

    def process_reply(self):
        lines = []
        while not self.rfile.closed:
            line = self.rfile.readline()
            if line.strip() == "EOResponse":
                break
            lines.append(line)
        data = "".join(lines).strip()
        if not data:
            return
        self.server.result_queue.put(json.loads(data))

    def respond(self, data, content_type):
        self.send_response(httplib.OK)
        self.send_header("Content-type", content_type)
        self.send_header("Content-Length", len(data))
        self.end_headers()
        self.wfile.write(data)

    # Just to make it quiet
    def log_message(self, format, *args):
        pass

def _find_chrome_in_registry():
    from _winreg import OpenKey, QueryValue, HKEY_CURRENT_USER
    path = r"Software\Microsoft\Windows\CurrentVersion\Uninstall\Google Chrome"

    try:
        key = OpenKey(HKEY_CURRENT_USER, path)
        install_dir = QueryValue(key, "InstallLocation")
    except OSError.WindowsError:
        return ""

    return os.path.join(install_dir, "chrome.exe")

def _is_win7():
    return sys.getwindowsversion()[0] == 6

def _default_windows_location():
    if _is_win7():
        appdata = os.environ["LOCALAPPDATA"]
    else:
        home = os.path.expanduser("~")
        appdata = os.path.join(home, "Local Settings", "Application Data")

    return os.path.join(appdata, "Google", "Chrome", "Application", "chrome.exe")

def _windows_chrome():
    return _find_chrome_in_registry() or _default_windows_location()

def _linux_chrome():
    locations = [
        "/usr/bin/google-chrome",
        "/usr/bin/chromium",
        "/usr/bin/chromium-browser",
    ]
    return _start_cmd(locations)

def _mac_chrome():
    locations = ["/Applications/Google Chrome.app/Contents/MacOS/Google Chrome"]
    import getpass
    locations.append(os.path.join("/Users/", getpass.getuser(), "Applications", "Google Chrome.app", "Contents", "MacOS", "Google Chrome"))
    return _start_cmd(locations)

def _start_cmd(locations):
    cmd = ""
    for cmd_path in locations:
        if os.path.exists(cmd_path):
            cmd = cmd_path
    if cmd == "":
        raise RuntimeError("Unable to find browser")
    return cmd

_BINARY = {
    "win32": _windows_chrome,
    "darwin": _mac_chrome,
    "linux2": _linux_chrome,
}

def chrome_exe():
    return _BINARY[sys.platform]()

def touch(filename):
    with open(filename, "ab"):
        pass

def _copy_zipped_extension(extension_zip):
    extension_dir = utils.unzip_to_temp_dir(extension_zip)
    return extension_dir

def create_extension_dir():
    extension_dir = _copy_zipped_extension("chrome-extension.zip")
    if extension_dir:
        return extension_dir
    extension_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), "chrome-extension.zip")
    extension_dir = _copy_zipped_extension(extension_dir)
    if extension_dir:
        return extension_dir
    path = tempfile.mkdtemp()
    # FIXME: Copied manually
    extdir = os.path.join(os.path.dirname(os.path.abspath(__file__)), "extension")
    if not os.path.isdir(extdir):
        extdir = os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), "extension")
        assert os.path.isdir(extdir), "can't find extension"
    # shutil.copytree need to create the directory
    shutil.rmtree(path)
    shutil.copytree(extdir, path)
    return path

def create_profile_dir():
    path = tempfile.mkdtemp()
    return path

PORT = 0

def on_error(func, path, exc_info):
    import stat
    if not os.access(path, os.W_OK):
        os.chmod(path, stat.S_IWUSR)
        func(path)
    else:
        pass

def run_chrome(extension_dir, profile_dir, port, untrusted_certificates, custom_args):
    command = [
        chrome_exe(),
        "--load-extension=%s" % extension_dir,
        "--user-data-dir=%s" % profile_dir,
        "--activate-on-launch",
        "--disable-hang-monitor",
        "--homepage=about:blank",
        "--no-first-run",
        "--disable-popup-blocking",
        "--disable-prompt-on-repost",
        "--no-default-browser-check",
        "http://localhost:%s/chromeCommandExecutor" % port,
        untrusted_certificates,
        custom_args,
    ]
    return subprocess.Popen(command)

def run_server(timeout=10):
    server = BaseHTTPServer.HTTPServer(("", 0), RequestHandler)
    server.command_queue = Queue.Queue()
    server.result_queue = Queue.Queue()
    t = threading.Thread(target=server.serve_forever)
    t.daemon = True
    t.start()
    start = time.time()
    while time.time() - start < timeout:
        try:
            urllib.urlopen("http://localhost:%s" % server.server_port)
            break
        except IOError:
            time.sleep(0.1)
    else:
        raise RemoteDriverServerException("Can't open server after %s seconds" % timeout)
    return server

class ChromeDriver:
    def __init__(self, custom_profile=None, untrusted_certificates=False, custom_args=""):
        self._server = None
        self._profile_dir = custom_profile
        self._extension_dir = None
        self._chrome = None
        self._untrusted_certificates = " --ignore-certificate-errors " if untrusted_certificates else ""
        self._custom_args = custom_args

    def start(self):
        self._extension_dir = create_extension_dir()
        self._profile_dir = create_profile_dir() if self._profile_dir is None else self._profile_dir
        self._server = run_server()
        self._chrome = run_chrome(self._extension_dir, self._profile_dir,
                                  self._server.server_port, self._untrusted_certificates,
                                  self._custom_args)

    def stop(self):
        try:
            os.kill(self._chrome.pid, signal.SIGTERM)
        except AttributeError:
            if platform.version() == "Windows":
                import ctypes
                PROCESS_TERMINATE = 1
                handle = ctypes.windll.kernel32.OpenProcess(PROCESS_TERMINATE, False, self._chrome.pid)
                ctypes.windll.kernel32.TerminateProcess(handle, -1)
                ctypes.windll.kernel32.CloseHandle(handle)
        self._chrome = None
        if self._server:
            self._server.server_close()
            self._server = None
        for path in (self._profile_dir, self._extension_dir):
            try:
                shutil.rmtree(path, ignore_errors=False, onerror=on_error)
            except OSError:
                pass

    def execute(self, command, params):
        to_send = params.copy()
        to_send["request"] = command
        self._server.command_queue.put(to_send)
        return self._server.result_queue.get()
    



# Copyright 2008-2009 WebDriver committers
# Copyright 2008-2009 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import base64
import httplib
from selenium.webdriver.common.exceptions import ErrorInResponseException
from selenium.webdriver.remote.command import Command
from selenium.webdriver.remote.webdriver import WebDriver as RemoteWebDriver
from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver.firefox.firefoxlauncher import FirefoxLauncher
from selenium.webdriver.firefox.firefox_profile import FirefoxProfile
from selenium.webdriver.firefox.extensionconnection import ExtensionConnection
import urllib2
import socket


class WebDriver(RemoteWebDriver):
    """The main interface to use for testing,
    which represents an idealised web browser."""

    def __init__(self, profile=None, timeout=30):
        """Creates a webdriver instance.

        Args:
          profile: a FirefoxProfile object (it can also be a profile name,
                   but the support for that may be removed in future, it is
                   recommended to pass in a FirefoxProfile object)
          timeout: the amount of time to wait for extension socket
        """
        port = self._free_port()
        self.browser = FirefoxLauncher()
        if type(profile) == str:
            # This is to be Backward compatible because we used to take a
            # profile name
            profile = FirefoxProfile(name=profile, port=port)
        if not profile:
            profile = FirefoxProfile(port=port)
        self.browser.launch_browser(profile)
        RemoteWebDriver.__init__(self,
            command_executor=ExtensionConnection(timeout),
            browser_name='firefox', platform='ANY', version='',
            javascript_enabled=True)

    def _free_port(self):
        port = 0
        free_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        free_socket.bind((socket.gethostname(), 0))
        port = free_socket.getsockname()[1]
        free_socket.close()
        return port

    def _execute(self, command, params=None):
        try:
            return RemoteWebDriver.execute(self, command, params)
        except ErrorInResponseException, e:
            # Legacy behavior: calling close() multiple times should not raise
            # an error
            if command != Command.CLOSE and command != Command.QUIT:
                raise e
        except urllib2.URLError, e:
            # Legacy behavior: calling quit() multiple times should not raise
            # an error
            if command != Command.QUIT:
                raise e
    
    def create_web_element(self, element_id):
        """Override from RemoteWebDriver to use firefox.WebElement."""
        return WebElement(self, element_id)

    def quit(self):
        """Quits the driver and close every associated window."""
        try:
            RemoteWebDriver.quit(self)
        except httplib.BadStatusLine:
            # Happens if Firefox shutsdown before we've read the response from
            # the socket.
            pass
        self.browser.kill()

    def save_screenshot(self, filename):
        """
        Gets the screenshot of the current window. Returns False if there is
        any IOError, else returns True. Use full paths in your filename.
        """
        png = self._execute(Command.SCREENSHOT)['value']
        try:
            f = open(filename, 'w')
            f.write(base64.decodestring(png))
            f.close()
        except IOError:
            return False
        finally:
            del png
        return True



#!/usr/bin/python
#
# Copyright 2011 Webdriver_name committers
# Copyright 2011 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from selenium.selenium import selenium
import selenium.webdriver.emulation.base as base
import selenium.webdriver.emulation.navigation as navigation

class DrivenSelenium(selenium):
  def __init__(self, driver, browserUrl):
	self.driver = driver
	self.browserUrl = browserUrl
	
  def start(self, browserConfigurationOptions=None):
    # This become a no-op. Should we blow up at this point?
    pass

  @property
  def webdriver(self):
	return self.driver
	
  def __getattribute__(self, attr):
    if not attr.startswith('_') and hasattr(navigation, attr):
      value = getattr(navigation, attr)
      if issubclass(value, base.BaseCommand):
        return value(self.driver, self.browserUrl)
	
    return super(DrivenSelenium, self).__getattribute__(attr)




import selenium.webdriver.emulation.base as base

class open(base.BaseCommand):
  def __call__(self, url):
    if url.find("://") == -1:
      if url.startswith('/'):
        toLoad = self.baseUrl + url
      else:
        toLoad = "%s/%s" % (self.baseUrl, url)
    else:
      toLoad = url
    self.driver.get(toLoad)
    
class go_back(base.BaseCommand):
  def __call__(self):
    self.driver.back()

class stop(base.BaseCommand):
  def __call__(self  , *args, **kwargs):
    self.driver.quit()



class BaseCommand(object):
  def __init__(self, driver, baseUrl):
    self.driver = driver
    if baseUrl.endswith('/'):
      self.baseUrl = baseUrl[:-1]
    else:
      self.baseUrl = baseUrl
    



# Copyright 2008-2009 WebDriver committers
# Copyright 2008-2009 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""WebElement implementation."""
from command import Command
from selenium.webdriver.common.by import By
from selenium.webdriver.common.exceptions import NoSuchAttributeException


class WebElement(object):
    """Represents an HTML element.

    Generally, all interesting operations to do with interacting with a page
    will be performed through this interface."""
    def __init__(self, parent, id_):
        self._parent = parent
        self._id = id_

    @property
    def tag_name(self):
        """Gets this element's tagName property."""
        return self.execute(Command.GET_ELEMENT_TAG_NAME)['value']

    @property
    def text(self):
        """Gets the text of the element."""
        return self.execute(Command.GET_ELEMENT_TEXT)['value']

    def click(self):
        """Clicks the element."""
        self.execute(Command.CLICK_ELEMENT)

    def submit(self):
        """Submits a form."""
        self.execute(Command.SUBMIT_ELEMENT)

    @property
    def value(self):
        """Gets the value of the element's value attribute."""
        return self.execute(Command.GET_ELEMENT_VALUE)['value']

    def clear(self):
        """Clears the text if it's a text entry element."""
        self.execute(Command.CLEAR_ELEMENT)

    def get_attribute(self, name):
        """Gets the attribute value."""
        resp = self.execute(Command.GET_ELEMENT_ATTRIBUTE, {'name': name})
        attributeValue = ''
        if resp['value'] is None:
            attributeValue = None
        else:
            attributeValue = str(resp['value'])
            if type(resp['value']) is bool:
                attributeValue = attributeValue.lower()

        return attributeValue

    def toggle(self):
        """Toggles the element state."""
        resp = self.execute(Command.TOGGLE_ELEMENT)
        return resp['value']

    def is_selected(self):
        """Whether the element is selected."""
        return self.execute(Command.IS_ELEMENT_SELECTED)['value']

    def select(self):
        """Selects an element."""
        self.execute(Command.SET_ELEMENT_SELECTED)

    def is_enabled(self):
        """Whether the element is enabled."""
        return self.execute(Command.IS_ELEMENT_ENABLED)['value']

    def find_element_by_id(self, id_):
        """Finds element by id."""
        return self.find_element(by=By.ID, value=id_)

    def find_elements_by_id(self, id_):
        return self.find_elements(by=By.ID, value=id_)

    def find_element_by_name(self, name):
        """Find element by name."""
        return self.find_element(by=By.NAME, value=name)

    def find_elements_by_name(self, name):
        return self.find_elements(by=By.NAME, value=name)

    def find_element_by_link_text(self, link_text):
        """Finds element by link text."""
        return self.find_element(by=By.LINK_TEXT, value=link_text)

    def find_elements_by_link_text(self, link_text):
        return self.find_elements(by=By.LINK_TEXT, value=link_text)

    def find_element_by_partial_link_text(self, link_text):
        return self.find_element(by=By.PARTIAL_LINK_TEXT, value=link_text)

    def find_elements_by_partial_link_text(self, link_text):
        return self.find_elements(by=By.PARTIAL_LINK_TEXT, value=link_text)

    def find_element_by_tag_name(self, name):
        return self.find_element(by=By.TAG_NAME, value=name)

    def find_elements_by_tag_name(self, name):
        return self.find_elements(by=By.TAG_NAME, value=name)

    def find_element_by_xpath(self, xpath):
        """Finds element by xpath."""
        return self.find_element(by=By.XPATH, value=xpath)

    def find_elements_by_xpath(self, xpath):
        """Finds elements within the elements by xpath."""
        return self.find_elements(by=By.XPATH, value=xpath)
    
    def find_element_by_class_name(self, name):
        """Finds an element by their class name."""
        return self.find_element(by=By.CLASS_NAME, value=name)

    def find_elements_by_class_name(self, name):
        """Finds elements by their class name."""
        return self.find_elements(by=By.CLASS_NAME, value=name)

    def find_element_by_css_selector(self, css_selector):
        """Find and return an element by CSS selector."""
        return self.find_element(by=By.CSS_SELECTOR, value=css_selector)
    
    def find_elements_by_css_selector(self, css_selector):
        """Find and return list of multiple elements by CSS selector."""
        return self.find_elements(by=By.CSS_SELECTOR, value=css_selector)

    def send_keys(self, *value):
        """Simulates typing into the element."""
        self.execute(Command.SEND_KEYS_TO_ELEMENT, {'value': value})

    # RenderedWebElement Items
    def is_displayed(self):
        """Whether the element would be visible to a user"""
        return self.execute(Command.IS_ELEMENT_DISPLAYED)['value']

    @property
    def size(self):
        """ Returns the size of the element """
        return self.execute(Command.GET_ELEMENT_SIZE)['value']

    def value_of_css_property(self, property_name):
        """ Returns the value of a CSS property """
        return self.execute(Command.GET_ELEMENT_VALUE_OF_CSS_PROPERTY,
                        {'propertyName': property_name})['value']

    @property
    def parent(self):
        return self._parent

    @property
    def id(self):
        return self._id

    # Private Methods
    def execute(self, command, params=None):
        """Executes a command against the underlying HTML element.

        Args:
          command: The name of the command to execute as a string.
          params: A dictionary of named parameters to send with the command.

        Returns:
          The command's JSON response loaded into a dictionary object.
        """
        if not params:
            params = {}
        params['id'] = self._id
        return self._parent.execute(command, params)

    def find_element(self, by=By.ID, value=None):
        return self.execute(Command.FIND_CHILD_ELEMENT,
                             {"using": by, "value": value})['value']

    def find_elements(self, by=By.ID, value=None):
        return self.execute(Command.FIND_CHILD_ELEMENTS,
                             {"using": by, "value": value})['value']



# Copyright 2010 WebDriver committers
# Copyright 2010 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from selenium.webdriver.common.exceptions import ElementNotSelectableException
from selenium.webdriver.common.exceptions import ElementNotVisibleException
from selenium.webdriver.common.exceptions import InvalidCookieDomainException
from selenium.webdriver.common.exceptions import InvalidElementStateException
from selenium.webdriver.common.exceptions import NoSuchElementException
from selenium.webdriver.common.exceptions import NoSuchFrameException
from selenium.webdriver.common.exceptions import NoSuchWindowException
from selenium.webdriver.common.exceptions import StaleElementReferenceException
from selenium.webdriver.common.exceptions import UnableToSetCookieException
from selenium.webdriver.common.exceptions import ErrorInResponseException
from selenium.webdriver.common.exceptions import TimeoutException
from selenium.webdriver.common.exceptions import WebDriverException


class ErrorCode(object):
    """Error codes defined in the WebDriver wire protocol."""
    # Keep in sync with org.openqa.selenium.remote.ErrorCodes and errorcodes.h
    SUCCESS = 0
    NO_SUCH_ELEMENT = 7
    NO_SUCH_FRAME = 8
    UNKNOWN_COMMAND = 9
    STALE_ELEMENT_REFERENCE = 10
    ELEMENT_NOT_VISIBLE = 11
    INVALID_ELEMENT_STATE = 12
    UNKNOWN_ERROR = 13
    ELEMENT_IS_NOT_SELECTABLE = 15
    JAVASCRIPT_ERROR = 17
    XPATH_LOOKUP_ERROR = 19
    NO_SUCH_WINDOW = 23
    INVALID_COOKIE_DOMAIN = 24
    UNABLE_TO_SET_COOKIE = 25
    TIMEOUT = 28


class ErrorHandler(object):
    """Handles errors returned by the WebDriver server."""
    def check_response(self, response):
        """
        Checks that a JSON response from the WebDriver does not have an error.
        Args:
        response - The JSON response from the WebDriver server as a dictionary
                object.
        Raises:
        If the response contains an error message.
        """
        status = response['status']
        if status == ErrorCode.SUCCESS:
            return
        exception_class = ErrorInResponseException
        if status == ErrorCode.NO_SUCH_ELEMENT:
            exception_class = NoSuchElementException
        elif status == ErrorCode.NO_SUCH_FRAME:
            exception_class = NoSuchFrameException
        elif status == ErrorCode.NO_SUCH_WINDOW:
            exception_class = NoSuchWindowException
        elif status == ErrorCode.STALE_ELEMENT_REFERENCE:
            exception_class = StaleElementReferenceException
        elif status == ErrorCode.ELEMENT_NOT_VISIBLE:
            exception_class = ElementNotVisibleException
        elif status == ErrorCode.INVALID_ELEMENT_STATE:
            exception_class = WebDriverException 
        elif status == ErrorCode.ELEMENT_IS_NOT_SELECTABLE:
            exception_class = ElementNotSelectableException
        elif status == ErrorCode.INVALID_COOKIE_DOMAIN:
            exception_class = WebDriverException 
        elif status == ErrorCode.UNABLE_TO_SET_COOKIE:
            exception_class = WebDriverException 
        elif status == ErrorCode.TIMEOUT:
            exception_class = TimeoutException
        elif status == ErrorCode.UNKNOWN_ERROR:
            exception_class = WebDriverException
        else:
            exception_class = WebDriverException
        value = response['value']
        if type(value) is str:
            if exception_class == ErrorInResponseException:
                raise exception_class(response, value)
            raise exception_class(value)
        message = ''
        if 'message' in value:
            message = value['message']
        # TODO:  What about 'screen' and 'stackTrace'?
        if exception_class == ErrorInResponseException:
            raise exception_class(response, message)
        raise exception_class(message)
